#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

ErrorStatus HSEStartUpStatus;

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

int main(void)
{
	EXTI_InitTypeDef EXTI_InitStructure;

	#ifdef DEBUG
  		debug();
	#endif

	DBGMCU_Config(DBGMCU_STOP, ENABLE);

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();
	
	// Linia 17 jako zdarzenie (RTC)
	EXTI_StructInit(&EXTI_InitStructure);
  	EXTI_InitStructure.EXTI_Line = EXTI_Line17;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Event;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);

  	PWR_BackupAccessCmd(ENABLE); // Zezwolenie na dostep do Backup domain

  	BKP_DeInit();

  	RCC_LSEConfig(RCC_LSE_ON);	// Wlacz LSE

  	while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET);	 // Czekaj, az wystartuje

  	RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);	 // LSE zrodlem sygn. zeg. dla RTC

  	RCC_RTCCLKCmd(ENABLE);	// Wlacz taktowanie RTC

  	RTC_WaitForSynchro(); 	// Czekaj na synchronizacje RTC z APB
  	RTC_WaitForLastTask();

  	RTC_SetPrescaler(32768);	// Zliczane beda impulsy co 1s
  	RTC_WaitForLastTask();

  	LCD_Init();
  	LCD_GoTo(0,0);
	delay_ms(2000);
  	while (1)
	{	
    	RTC_SetAlarm(RTC_GetCounter()+ 10);	 // Wybudzenie co 10s
        RTC_WaitForLastTask();	

		LCD_Clear();
		LCD_GoTo(0,0);
		LCD_SendText("Tryb uspienia");		

    	PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFE);

		// *** Ponowne wlaczenie PLL i HSE ***
		RCC_HSEConfig(RCC_HSE_ON);
		HSEStartUpStatus = RCC_WaitForHSEStartUp();

		if(HSEStartUpStatus == SUCCESS)
		{
			RCC_PLLCmd(ENABLE);
			while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);
			RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
			while(RCC_GetSYSCLKSource() != 0x08);
		}
		// *** HSE i PLL wlaczone ***

		LCD_Clear();
		LCD_GoTo(0,0);
		LCD_SendText(" Pracuje ");

		// Mignij LED8
		GPIO_ResetBits(GPIOB, GPIO_Pin_15);
		delay_ms(1000);
		GPIO_SetBits(GPIOB, GPIO_Pin_15);
		delay_ms(1000);
	}
}

void RCC_Conf(void)
{  
  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/4
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
  	}

  	// Wlacz taktowanie GPIOA
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// Konfiguracja PB15
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
