#include "stm32f10x.h"
#include "lcd4bit.h"

void GPIO_Conf(void);
void RCC_Conf(void);
void NVIC_Conf(void);

int main(void)
{
#ifdef DEBUG
  debug();
#endif
  	RCC_Conf(); // Standardowa konfiguracja sygnalow zegarowych i taktowania peryferiow
  	GPIO_Conf();

	DBGMCU_Config(DBGMCU_STANDBY, ENABLE);

  	// Wlacz obsluge wyprowadzenia WKUP (PA0)
  	PWR_WakeUpPinCmd(ENABLE);

  	// Zezwolenie na dostep do "Backup domain"
  	PWR_BackupAccessCmd(ENABLE);

	LCD_Init();
	LCD_Clear();

  	// Czy flaga trybu czuwania ustawiona?
  	if(PWR_GetFlagStatus(PWR_FLAG_SB) != RESET)
  	{
	// Jesli tak, to system budzi sie z trybu standby
		LCD_GoTo(1,0);
		LCD_SendText("Bylo STANDBY"); 
		
		// Wyczysc flage PWR_FLAG_SB
		PWR_ClearFlag(PWR_FLAG_SB);
		
		// Czekaj na synchronizacje z APB1
		RTC_WaitForSynchro();
	}
	else
	{
	//System startuje normalnie, wiec nalezy skonfigurowac RTC	

		BKP_DeInit();
		  
		// Wlacz wolny zewnetrzny oscylator
		RCC_LSEConfig(RCC_LSE_ON);
		// Czekaj na poprawny sygnal
		while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET);
		
		// LSE bedzie zrodlem sygnalu zegarowego dla RTC
		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
		
		// Wlacz taktowanie RTC
		RCC_RTCCLKCmd(ENABLE);

		// Czekaj na synchronizacje z APB1
		RTC_WaitForSynchro();
		RTC_WaitForLastTask();
		
		// RTC bedzie zliczal pojedyncze sekundy
		RTC_SetPrescaler(32768);
		// Czekaj na wykonanie polecenia  
		RTC_WaitForLastTask();
	}

	NVIC_Conf();

	LCD_GoTo(0,0);
	LCD_SendText("Pracuje");

 	while (1)
  	{
		if(!GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1))
		{
			LCD_Clear();
			LCD_GoTo(0,0);
			LCD_SendText("StandBy");

			// Wlacz tryb czuwania (standby mode)
			PWR_EnterSTANDBYMode();
		}
	}
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
  	}
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// PA1 jako wejscie plywajace (SW1)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  // Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  // W przeciwnym wypadku ustaw na 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
