/* Kod na podsawie przykladow dostarczanych wraz ze zrodlami FreeRTOS */

#include <stdio.h>

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "stm32f10x_it.h"
#include "GPIO_LED.h"
#include "TaskLED.h"
#include "semphr.h"
#include "taskSW.h"

EXTI_InitTypeDef EXTI_InitStructure;
GPIO_InitTypeDef GPIO_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

/* Task priorities. */
#define mainCHECK_TASK_PRIORITY				( tskIDLE_PRIORITY + 3 )

/* The check task uses the sprintf function so requires a little more stack. */
#define mainCHECK_TASK_STACK_SIZE			( configMINIMAL_STACK_SIZE + 50 )

/* The time between cycles of the 'check' task. */
#define mainCHECK_DELAY						( ( portTickType ) 5000 / portTICK_RATE_MS )

/* The number of nano seconds between each processor clock. */
#define mainNS_PER_CLOCK ( ( unsigned portLONG ) ( ( 1.0 / ( double ) configCPU_CLOCK_HZ ) * 1000000000.0 ) )

/*-----------------------------------------------------------*/

#define mainFLASH_TASK_PRIORITY				( tskIDLE_PRIORITY + 1 )

xSemaphoreHandle xSemaphoreSW0 = NULL;
xSemaphoreHandle xSemaphoreSW1 = NULL;
xSemaphoreHandle xSemaphoreSW2 = NULL;

static void prvSetupHardware( void );

void EXTI0_IRQHandler(void)
{
	static portBASE_TYPE xHigherPriorityTaskWoken;
	xHigherPriorityTaskWoken = pdFALSE;
  	if(EXTI_GetITStatus(EXTI_Line0) != RESET)
  	{
   		xSemaphoreGiveFromISR(xSemaphoreSW0,&xHigherPriorityTaskWoken);
    	
    	EXTI_ClearITPendingBit(EXTI_Line0);
  	}
}

void EXTI1_IRQHandler(void)
{
	static portBASE_TYPE xHigherPriorityTaskWoken;
	xHigherPriorityTaskWoken = pdFALSE;
  	if(EXTI_GetITStatus(EXTI_Line1) != RESET)
  	{
   		xSemaphoreGiveFromISR(xSemaphoreSW1,&xHigherPriorityTaskWoken);
    	// Czyszczenie flagi przerwania
    	EXTI_ClearITPendingBit(EXTI_Line1);
  	}
}

void EXTI2_IRQHandler(void)
{
	static portBASE_TYPE xHigherPriorityTaskWoken;
	xHigherPriorityTaskWoken = pdFALSE;
  	if(EXTI_GetITStatus(EXTI_Line2) != RESET)
  	{
   		xSemaphoreGiveFromISR(xSemaphoreSW2,&xHigherPriorityTaskWoken);
    	// Czyszczenie flagi przerwania
    	EXTI_ClearITPendingBit(EXTI_Line2);
  	}
}


int main( void )
{
#ifdef DEBUG
  debug();
#endif

	// Konfiguracja sprzetu
	prvSetupHardware();
  
  	// Konfiguracja i wlaczenie przerwania
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQChannel;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line0
  	EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);


  	// Konfiguracja i wlaczenie przerwania
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQChannel;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource1);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line1
  	EXTI_InitStructure.EXTI_Line = EXTI_Line1;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);


  	// Konfiguracja i wlaczenie przerwania
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI2_IRQChannel;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource2);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line2
  	EXTI_InitStructure.EXTI_Line = EXTI_Line2;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);


  	// Konfiguracja i wlaczenie przerwania
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI3_IRQChannel;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource3);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line3
  	EXTI_InitStructure.EXTI_Line = EXTI_Line3;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);


	// Uruchomienie zadan
	vStartSWTasks( mainFLASH_TASK_PRIORITY );	
	
	// Uruchomienie planisty
	vTaskStartScheduler();

	return 0;
}
/*-----------------------------------------------------------*/

static void prvSetupHardware( void )
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Start with the clocks in their expected state. */
	RCC_DeInit();

	/* Enable HSE (high speed external clock). */
	RCC_HSEConfig( RCC_HSE_ON );

	/* Wait till HSE is ready. */
	while( RCC_GetFlagStatus( RCC_FLAG_HSERDY ) == RESET );

	/* 2 wait states required on the flash. */
	*( ( unsigned portLONG * ) 0x40022000 ) = 0x02;

	/* HCLK = SYSCLK */
	RCC_HCLKConfig( RCC_SYSCLK_Div1 );

	/* PCLK2 = HCLK */
	RCC_PCLK2Config( RCC_HCLK_Div1 );

	/* PCLK1 = HCLK/2 */
	RCC_PCLK1Config( RCC_HCLK_Div2 );

	/* PLLCLK = 8MHz * 9 = 72 MHz. */
	RCC_PLLConfig( RCC_PLLSource_HSE_Div1, RCC_PLLMul_9 );

	/* Enable PLL. */
	RCC_PLLCmd( ENABLE );

	/* Wait till PLL is ready. */
	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

	/* Select PLL as system clock source. */
	RCC_SYSCLKConfig( RCC_SYSCLKSource_PLLCLK );

	/* Wait till PLL is used as system clock source. */
	while( RCC_GetSYSCLKSource() != 0x08 );

	/* Enable GPIOA, GPIOB, GPIOC, GPIOD, GPIOE and AFIO clocks */
	RCC_APB2PeriphClockCmd(	RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |RCC_APB2Periph_GPIOC
							| RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOE | RCC_APB2Periph_AFIO, ENABLE );

	/* SPI2 Periph clock enable */
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_SPI2, ENABLE );

	/* Set the Vector Table base address at 0x08000000 */
	NVIC_SetVectorTable( NVIC_VectTab_FLASH, 0x0 );

	NVIC_PriorityGroupConfig( NVIC_PriorityGroup_4 );
	
	/* Configure HCLK clock as SysTick clock source. */
	SysTick_CLKSourceConfig( SysTick_CLKSource_HCLK );
	
	GPIO_InitStructure.GPIO_Pin =  	GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | 
									GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOB, &GPIO_InitStructure );
}
/*-----------------------------------------------------------*/

#ifdef  DEBUG
void assert_failed( unsigned portCHAR* pcFile, unsigned portLONG ulLine )
{
	for( ;; );
}
#endif
