/* Three tasks are created, each of which flash an LED at a different rate.  The first 
 * LED flashes every 200ms, the second every 400ms, the third every 600ms.
 *
 * The LED flash tasks provide instant visual feedback.  They show that the scheduler 
 * is still operational*/

#include <stdlib.h>

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

/* Demo program include files. */
#include "GPIO_LED.h"
#include "taskLED.h"

#define ledSTACK_SIZE		configMINIMAL_STACK_SIZE
#define SEM_STACK_SIZE		configMINIMAL_STACK_SIZE
#define ledNUMBER_OF_LEDS	( 6 )
#define ledFLASH_RATE_BASE	( ( portTickType ) 333 )

void vTaskLED1(void*);
void vTaskLED2(void*);
void vTaskLED3(void*);


void vStartLEDTasks(unsigned portBASE_TYPE uxPriority)
{
	xTaskHandle xHandleTaskLED1, xHandleTaskLED2, xHandleTaskLED3;

	// Tworzenie zadania migania LED1 z f = 1Hz
	xTaskCreate(vTaskLED1, ( signed portCHAR * ) "LED1", 
				ledSTACK_SIZE, NULL, uxPriority, &xHandleTaskLED1);
	// Tworzenie zadania migania LED1 z f = 0.5Hz
	xTaskCreate(vTaskLED2, ( signed portCHAR * ) "LED2", 
				ledSTACK_SIZE, NULL, uxPriority, &xHandleTaskLED2);
	// Tworzenie zadania migania LED1 z f = 0.25Hz
	xTaskCreate(vTaskLED3, ( signed portCHAR * ) "LED3", 
				ledSTACK_SIZE, NULL, uxPriority, &xHandleTaskLED3);

}

void vTaskLED1(void * pvParameters)
{
	portTickType xLastFlashTime;
	
	// Odczytanie stanu licznika systemowego  
	xLastFlashTime = xTaskGetTickCount();
	// Nieskonczona petla zadania
	for(;;)
	{
		// Wprowadzenie opoznienia 500ms
		vTaskDelayUntil( &xLastFlashTime, 500/portTICK_RATE_MS );
		// Zmiana stanu wyprowadzenia PB8 (LED1) na przeciwny
		vhToggleLED1();
	}
}

void vTaskLED2(void * pvParameters)
{
	portTickType xLastFlashTime;
	
	// Odczytanie stanu licznika systemowego  
	xLastFlashTime = xTaskGetTickCount();
	// Nieskonczona petla zadania
	for(;;)
	{
		// Wprowadzenie opoznienia 500ms
		vTaskDelayUntil( &xLastFlashTime, 1000/portTICK_RATE_MS );
		// Zmiana stanu wyprowadzenia PB8 (LED1) na przeciwny
		vhToggleLED2();
	}
}

void vTaskLED3(void * pvParameters)
{
	portTickType xLastFlashTime;
	
	// Odczytanie stanu licznika systemowego  
	xLastFlashTime = xTaskGetTickCount();
	// Nieskonczona petla zadania
	for(;;)
	{
		// Wprowadzenie opoznienia 500ms
		vTaskDelayUntil( &xLastFlashTime, 2000/portTICK_RATE_MS );
		// Zmiana stanu wyprowadzenia PB8 (LED1) na przeciwny
		vhToggleLED3();
	}
}



 /*
void vTaskLED(void * pvParameters)
{
	// Nieskonczona petla zadania
	for(;;)
	{
		// Wprowadzenie opoznienia 500ms
		vTaskDelay(500/portTICK_RATE_MS);
		// Zmiana stanu wyprowadzenia PB8 (LED1) na przeciwny
		vhToggleLED();
	}
}
   */
 /*
static portTASK_FUNCTION( vLEDFlashTask, pvParameters )
{
portTickType xFlashRate, xLastFlashTime;
unsigned portBASE_TYPE uxLED;

	// The parameters are not used.
	( void ) pvParameters;

	// Calculate the LED and flash rate.
	portENTER_CRITICAL();
	{
		// See which of the eight LED's we should use.
		uxLED = uxFlashTaskNumber;

		// Update so the next task uses the next LED.
		uxFlashTaskNumber++;
	}
	portEXIT_CRITICAL();

	xFlashRate = ledFLASH_RATE_BASE + ( ledFLASH_RATE_BASE * ( portTickType ) uxLED );
	xFlashRate /= portTICK_RATE_MS;

	// We will turn the LED on and off again in the delay period, so each delay is only half the total period.
	xFlashRate /= ( portTickType ) 2;

	// We need to initialise xLastFlashTime prior to the first call to vTaskDelayUntil().
	xLastFlashTime = xTaskGetTickCount();

	for(;;)
	{
		// Delay for half the flash period then turn the LED on. 
		vTaskDelayUntil( &xLastFlashTime, xFlashRate );
		vParTestToggleLED( uxLED );

		// Delay for half the flash period then turn the LED off.
		vTaskDelayUntil( &xLastFlashTime, xFlashRate );
		vParTestToggleLED( uxLED );
	}
} //lint !e715 !e818 !e830 Function definition must be standard for task creation.
	 */
