#include "stm32f10x.h"

ErrorStatus HSEStartUpStatus;

u16 i = 0;

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

void delay_ms(int ms)
{
	int i, tms;
	tms = 9000*ms;
	for(i=0;i<tms;i++);
}

int main(void)
{
#ifdef DEBUG
  debug();
#endif

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();
  
  	// Wlaczenie wymaganych sygnalow zegarowych
  	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

  	// Zezwolenie na zapis do chronionych rejestrow
  	PWR_BackupAccessCmd(ENABLE);

  	// Wyzerowanie flagi
  	BKP_ClearFlag();

  	// Sprawdzenie, czy uC uruchamia sie po wlaczeniu zasilania
  	if(RCC_GetFlagStatus(RCC_FLAG_PORRST) != RESET)
  	{
    	// Wyczyszczenie flagi resetu
    	RCC_ClearFlag();
   
    	// Sprawdzenie, czy rejestry chronione poprawie zapisane
    	if(BKP_ReadBackupRegister(BKP_DR1) == 0x00FF)
     	 	// Zapal diode LED2 - rejestry chronione OK
     	 	GPIO_SetBits(GPIOB, GPIO_Pin_9);
    	else
  	    	// Zapal diode LED1 - nastapilo zresetowanie rejestrow chronionych
  	    	GPIO_SetBits(GPIOB, GPIO_Pin_8);
  	}  
  	while (1)
  	{ 
		delay_ms(2000);
  		BKP_WriteBackupRegister(BKP_DR2, i++);   
  	}
}

void RCC_Conf(void)
{
  // Reset ustawien RCC
  RCC_DeInit();

  // Wlacz HSE
  RCC_HSEConfig(RCC_HSE_ON);

  // Czekaj az HSE bedzie gotowy
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    // zwloka dla pamieci Flash
    FLASH_SetLatency(FLASH_Latency_2);
 
    // HCLK = SYSCLK
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    // PCLK2 = HCLK
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    // PCLK1 = HCLK/2
    RCC_PCLK1Config(RCC_HCLK_Div2);

    // PLLCLK = 8MHz * 9 = 72 MHz
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    // Wlacz PLL
    RCC_PLLCmd(ENABLE);

    // Czekaj az PLL poprawnie sie uruchomi
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    // PLL bedzie zrodlem sygnalu zegarowego
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    // Czekaj az PLL bedzie sygnalem zegarowym systemu
    while(RCC_GetSYSCLKSource() != 0x08);
  }

  // Wlacz taktowanie GPIOB
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
}

void GPIO_Conf(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  // Wlacz obsluge diod LED (LD1 i LD2)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
  // Tablica wektorow przerwan 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
