#include "stm32f10x.h"

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

int main(void)
{
#ifdef DEBUG
  debug();
#endif

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();

  	if (BKP_ReadBackupRegister(BKP_DR1) != 0x1111)
  	{
  		// Rejestry chronione zresetowane
		// Zapal LED1
		GPIO_SetBits(GPIOB, GPIO_Pin_8);

    	// Konfiguracja RTC:

  		// Zezwolenie na dostep do rejestrow chronionych przez utrata
  		PWR_BackupAccessCmd(ENABLE);

  		// Reset rejestrow chronionych
  		BKP_DeInit();

		// Wartosc kontrolna wpisana do rejestru BKP_DR1
    	BKP_WriteBackupRegister(BKP_DR1, 0x1111);

  		// Wlaczenie oscylatora LSE
  		RCC_LSEConfig(RCC_LSE_ON);
  		while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET);

  		// LSE bedzie zrodlem sygnalu zegarowego dla RTC
  		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

  		// Wlaczenie taktowania RTC
  		RCC_RTCCLKCmd(ENABLE);

  		// Czekaj na synchronizacje
  		RTC_WaitForSynchro();

  		// Czekaj na zakonczenie operacji
  		RTC_WaitForLastTask();

  		// Wlacz przerwanie od RTC (co 1s)
  		RTC_ITConfig(RTC_IT_SEC, ENABLE);

  		RTC_WaitForLastTask();

  		// Okres RTC bedzie wynosil 1s
  		RTC_SetPrescaler(32767); // RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1)

  		RTC_WaitForLastTask();
  	}
  	else
  	{
		// Rejestry chronione nie zostaly zresetowane
		// Zapal LED2
		GPIO_SetBits(GPIOB, GPIO_Pin_9);

    	// Czekaj na synchronizacje
    	RTC_WaitForSynchro();

    	// Wlacz przerwanie od RTC (co 1s)
    	RTC_ITConfig(RTC_IT_SEC, ENABLE);
    	// Czekaj na zakonczenie operacji
    	RTC_WaitForLastTask();
  	}

  	// Wyczysc flagi RCC
  	RCC_ClearFlag();

  	while(1);
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
  	}
  	// Wlacz taktowanie GPIOB, PWR i BKP
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
}

void NVIC_Conf(void)
{
  	NVIC_InitTypeDef NVIC_InitStructure;

#ifdef  VECT_TAB_RAM
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif

  	// Ustalenie grupy przerwan
  	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  	// Wlacz przerwanie od RTC
  	NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// Wlacz obsluge wszyskich diod LED (LD1 do LD8)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);

  	GPIO_ResetBits(GPIOB, GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15);
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
