#include "stm32f10x.h"

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);
void EXTI_Conf(void);

int main(void)
{
#ifdef DEBUG
  debug();
#endif

  	RCC_Conf();
  	GPIO_Conf();
	
  	// Sprawdzenie, czy system rozpoczyna prace po resecie od IWDG
  	if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) != RESET)
  	{
    	// Jesli tak to zapal diode LED1
    	GPIO_SetBits(GPIOB, GPIO_Pin_8);

    	// Wyczysc fagi RCC
    	RCC_ClearFlag();
  	}
  	else
  	{
    	// Jesli inne zrodlo resetu, to zgas diode LED1
    	GPIO_ResetBits(GPIOB, GPIO_Pin_8);
  	}

  	NVIC_Conf();
	EXTI_Conf();	  // Nacisniecie SW0 spowoduje zablokowanie przerwan od SysTick

  	// Zezwolenie za zapis rejestrw IWDG
  	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

  	// IWDG taktowany zegarem: 40kHz/128 = 312Hz
  	IWDG_SetPrescaler(IWDG_Prescaler_128);

  	// Przepelnienie IWDG po okolo 13s
  	IWDG_SetReload(0xFFF);

  	// Przeladowanie IWDG
  	IWDG_ReloadCounter();

  	// Wlaczenie IWDG i LSI
  	IWDG_Enable();

  	while(1);
}

void EXTI_Conf(void)
{
  	EXTI_InitTypeDef EXTI_InitStructure;

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line0
  	EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
  	}
  	// Wlacz taktowanie GPIOB, PWR i BKP
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOA, ENABLE);
}

void NVIC_Conf(void)
{
  	NVIC_InitTypeDef NVIC_InitStructure;

#ifdef  VECT_TAB_RAM
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif

  	// Wybranie grupy priorytetow
  	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  	// Wlacz przerwanie od PA0
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// Wlacz obsluge wszyskich diod LED (LD1 do LD8)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);

  	GPIO_ResetBits(GPIOB, GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15);

	// Konfiguracja PA0
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
