#include "stm32f10x.h"

void RCC_Conf(void);
void NVIC_Conf(void);

ErrorStatus HSEStartUpStatus;

__asm f_asm(void)
{
	;konfiguracja GPIOB
	LDR R0, =0x40021018 ;adres rejestru RCC_APB2ENR
	LDR R1, =0x00000008	;wlacz zegar dla GPIOB
	STR R1, [R0]		;wpisz do pamieci pod adres z R0 wartosc z R1
	LDR R0, =0x40010C04	;adres rejestru GPIOC_CRH
	LDR R1, =0x33333333	;wszystkie jako push-pull
	STR R1, [R0]		;wpisz do pamieci pod adres z R0 wartosc z R1
	;koniec konfiguracja GPIOB

	;ustawienie pinow PB8,PB9,PB10,PB11 w stan wysoki (zapalenie LED)
	;piny PB12,PB13,PB14,PB15 w stan niski
	LDR R0, =0x40010C10 ;adres rejestru GPIO_BSRR
	LDR R1, =0xF0000F00	;ustawienie stanu wysokiego na pinach PB8 do PB11,
						;a niskiego na PB12 do PB15
	STR R1, [R0]		;wpisz do pamieci pod adres z R0 wartosc z R1

	BX LR			  	;powrot do miejsca wywolania
}

int main(void)
{
#ifdef DEBUG
  debug();
#endif	 
  	RCC_Conf();   
  	NVIC_Conf();

	f_asm();
  
	while(1);	
}

void RCC_Conf(void)
{  
  ErrorStatus HSEStartUpStatus; 

  // Reset ustawien RCC
  RCC_DeInit();

  // Wlacz HSE
  RCC_HSEConfig(RCC_HSE_ON);

  // Czekaj za HSE bedzie gotowy
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    // zwloka dla pamieci Flash
    FLASH_SetLatency(FLASH_Latency_2);
 	
    // HCLK = SYSCLK
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    // PCLK2 = HCLK
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    // PCLK1 = HCLK/2
    RCC_PCLK1Config(RCC_HCLK_Div2);

    // PLLCLK = 8MHz * 9 = 72 MHz
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    // Wlacz PLL 
    RCC_PLLCmd(ENABLE);

    // Czekaj az PLL poprawnie sie uruchomi
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    // PLL bedzie zrodlem sygnalu zegarowego
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    // Czekaj az PLL bedzie sygnalem zegarowym systemu
    while(RCC_GetSYSCLKSource() != 0x08);
  }
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  // Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  // W przeciwnym wypadku ustaw na 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
