#include "stm32f10x.h"

EXTI_InitTypeDef EXTI_InitStructure;
GPIO_InitTypeDef GPIO_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

void RCC_Conf(void);
void NVIC_Conf(void);

int main(void)
{
#ifdef DEBUG
  	debug();
#endif
	
  	RCC_Conf();
  	NVIC_Conf();

  	// Wybranie grupy priorytetw
  	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
  
  	// Konfiguracja i wlaczenie przerwania
  	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	// Konfiguracja wyprowadzenia PB9 jako wyjscie push-pull (LD1)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);
    
  	// Konfiguracja wyprowadzenia PA0 jako wejscie plywajace
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line0
  	EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);
     
  	while (1);
}

void RCC_Conf(void)
{
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
   
  		// Wlaczenie sygnalu zegarowego dla GPIOB, GPIOC i funkcji alternatywnych
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);
  	}
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  	// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  	// W przeciwnym wypadku ustaw na 0x08000000
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
