#include "stm32f10x.h"

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

int main(void)
{
#ifdef DEBUG
  	debug();
#endif

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();
							 
  	// Konfiguracja Timera 3
  	TIM_TimeBaseStructure.TIM_Period = 999;
  	TIM_TimeBaseStructure.TIM_Prescaler = 0;
  	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);
  
  	// Konfiguracja kanalu 1
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  	TIM_OCInitStructure.TIM_Pulse = 950;
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  	TIM_OC1Init(TIM3, &TIM_OCInitStructure);

  	TIM_OC1PreloadConfig(TIM3, TIM_OCPreload_Enable);

  	// Konfiguracja kanalu 2
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  	TIM_OCInitStructure.TIM_Pulse = 350;
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  	TIM_OC2Init(TIM3, &TIM_OCInitStructure);

  	TIM_OC2PreloadConfig(TIM3, TIM_OCPreload_Enable);
	  
  	// Konfiguracja kanalu 3
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;							     
  	TIM_OCInitStructure.TIM_Pulse = 250;
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  	TIM_OC3Init(TIM3, &TIM_OCInitStructure);

  	TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Enable);

  	// Konfiguracja kanalu 4
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  	TIM_OCInitStructure.TIM_Pulse = 100;
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  	TIM_OC4Init(TIM3, &TIM_OCInitStructure);

  	TIM_OC4PreloadConfig(TIM3, TIM_OCPreload_Enable);
					
  	TIM_ARRPreloadConfig(TIM3, ENABLE);
		   
  	// Wlaczenie Timera 3 
  	TIM_Cmd(TIM3, ENABLE);
						
  	while(1);
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

  		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
  	}
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	GPIO_PinRemapConfig(GPIO_FullRemap_TIM3, ENABLE);

  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOC, &GPIO_InitStructure);			  
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  	// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  	// W przeciwnym wypadku ustaw na 0x08000000
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
