#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

__IO uint16_t cnt;
uint16_t time = 0;								   
char t[8]; 

int main(void)
{
	#ifdef DEBUG
  		debug();
	#endif

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();

  	// Konfiguracja ukladu podstawy czasu
  	TIM_TimeBaseStructure.TIM_Period = 65535;
  	TIM_TimeBaseStructure.TIM_Prescaler = 65535; //fclk = 72MHz/65535 = 1.098kHz
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  	// Konfiguracja kanalu 2 jako "input capture"
  	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;

  	// Bedzie reagowal na zbocze narastajace
  	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;  
  	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;

  	// Nie uzywamy preskalera, kazde zdarzenie jest uwzgledniane
  	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
	// Nie uzywamy filtra	  
  	TIM_ICInitStructure.TIM_ICFilter = 0;

  	TIM_ICInit(TIM2, &TIM_ICInitStructure);

  	TIM_ITConfig(TIM2, TIM_IT_CC2, ENABLE);  // Wlaczenie przerwania od TIM2

  	LCD_Init();
  	LCD_GoTo(0,0);

  	// Wlaczenie timera
  	TIM_Cmd(TIM2, ENABLE);

  	while (1)
  	{
  		LCD_GoTo(0,0);
		LCD_Clear();
		time = cnt*0.91;   //time = cnt/fclk = cnt/1.098kHz [ms]
  		sprintf(t, "%d ms", time);
		LCD_SendText("t = "); 
		LCD_SendText((uint8_t*)t); 
		delay_ms(15);  	// Odswiezanie - odczekaj mniej wiecej 15ms
  	}
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/4
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

  		// Wlacz taktowanie GPIOB i TIM2
  		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
  	}
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// PA1 wyjsciem drugiego kanalu TIM2
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif

  	// Wlacz przerwanie od TIM2
  	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
