#include "stm32f10x.h"

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef TIM_ICInitStructure;

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

int main(void)
{
	#ifdef DEBUG
  		debug();
	#endif

  	RCC_Conf(); 
	NVIC_Conf(); 
	GPIO_Conf();

  	// ***** Konfiguracja TIM1 jako master dla TIM2 *****

  	// ustawienie paramterow ukladu podstawy czasu
  	TIM_TimeBaseStructure.TIM_Period = 19999;          
  	TIM_TimeBaseStructure.TIM_Prescaler = 35999;       
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;    
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up; 
    
  	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);
  
  	// Kanal 1 w trybie aktywnym
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Active;          
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;  
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;         
  	TIM_OCInitStructure.TIM_Pulse = 1;  
  
  	TIM_OC1Init(TIM1, &TIM_OCInitStructure);

  	// Kanal 1 TIM1 jako wyjscie wyzwalajace
  	TIM_SelectOutputTrigger(TIM1, TIM_TRGOSource_OC1);

	// ***** Konfiguracja TIM2 jako slave dla TIM1 *****

  	// ustawienie paramterow ukladu podstawy czasu
  	TIM_TimeBaseStructure.TIM_Period = 719;          
  	TIM_TimeBaseStructure.TIM_Prescaler = 0;       
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;    
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up; 
  
  	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  	// Kanal 3 w trybie Toggle
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Toggle;          
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;  
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;          
  	TIM_OCInitStructure.TIM_Pulse = 5;  
  
  	TIM_OC3Init(TIM2, &TIM_OCInitStructure);
  
  	// TIM2 jako slave, sygnal z TIM1 sygnalem zegarowym dla TIM2
  	TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_External1);

  	// Wybranie wejscia wyzwalania/zegarowego
  	TIM_SelectInputTrigger(TIM2, TIM_TS_ITR0);

  	// Wlaczenie obydwu timerow
  	TIM_Cmd(TIM2, ENABLE);
  	TIM_Cmd(TIM1, ENABLE);

  	while (1);
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

  		// Wlacz taktowanie GPIOB, TIM1 i TIM2
  		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
  	}
}

void GPIO_Conf(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;

  // Wlaczenie remappingu TIM2
  GPIO_PinRemapConfig(GPIO_FullRemap_TIM2, ENABLE);

  // Po remappingu kanal 3 TIM2 na PB10
  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;

  GPIO_Init(GPIOB, &GPIO_InitStruct);	
}

void NVIC_Conf(void)
{
	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
