#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>
    
void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

ADC_InitTypeDef ADC_InitStructure;

uint16_t ADC1Val = 0;
char wynik[7];

int main(void)
{
	#ifdef DEBUG
  		debug();
	#endif

	RCC_Conf();		// Konfiguracja sygnalow zegarowych
	NVIC_Conf(); 	// Konfiguracja NVIC
	GPIO_Conf();	// Konfiguracja wyprowadzen
	LCD_Init();		// Inicjalizacja LCD
  
	// Jeden przetwornik, pracujacy niezaleznie   
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	// Pomiar jednego kanalu, wylacz opcje skanowania
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	// Wlacz pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	// Nie bedzie wyzwalania zewnetrznego
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	// Dane wyrownane do prawej - znaczacych bedzie 12 mlodszych bitow
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	// Jeden kanal
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	// Inicjuj przetwornik
	ADC_Init(ADC1, &ADC_InitStructure);
	// Grupa regularna, czas probkowania 28,5 cykla czyli???
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_71Cycles5);    

	// Wlacz ADC1
	ADC_Cmd(ADC1, ENABLE);								

	// Resetuj rejestry kalibracyjne  
	ADC_ResetCalibration(ADC1);
	// Czekaj, az skonczy resetowac
	while(ADC_GetResetCalibrationStatus(ADC1));

	// Start kalibracji ADC1
	ADC_StartCalibration(ADC1);
	// Czekaj na zakonczenie kalibracji ADC1
	while(ADC_GetCalibrationStatus(ADC1));

	// Start przetwarzania
	ADC_SoftwareStartConvCmd(ADC1, ENABLE); 

	while (1)
	{
		// Wartosc z ADC1 
		ADC1Val = ADC_GetConversionValue(ADC1) * 0.807;   
		// Do tablicy 'wynik' wpisz sformatowany rezultat pomiaru
		// Dzielenie calkowite wyluskuje wartosc w woltach
		// natomiast dzielenie modulo pozwala uzyskac wartosc po przecinku
		sprintf(wynik, "%d,%03d V", ADC1Val / 1000, ADC1Val % 1000);
		// Wyswietl wynik
		LCD_Clear();
		LCD_GoTo(0,0);
		LCD_SendText("NAPIECIE NA P1:");
		LCD_GoTo(1,3);
		LCD_SendText((uint8_t*)wynik);
		// Aktualizacja wyswietlacza co okolo 0.5s
		delay_ms(500);
	}
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/4
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);
  	}

  	// Wlacz taktowanie GPIOC i ADC1
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_GPIOC, ENABLE);
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// PC4 jako analogowe wejscie
  	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOC, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif/
