#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

#define ADC1_DR_Address    ((uint32_t)0x4001244C)

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);
void DMA_Conf(void);

ADC_InitTypeDef ADC_InitStruct;
int temperatura, napiecie;
char wynik_temperatura[9];
char wynik_napiecie[8];
__IO uint16_t ADCVal[2];

int main(void)
{	
	#ifdef DEBUG
  		debug();
	#endif	
	
  	RCC_Conf();   
  	NVIC_Conf();
	GPIO_Conf();
	DMA_Conf();

	// Jeden przetwornik, pracujacy niezaleznie   
	ADC_InitStruct.ADC_Mode = ADC_Mode_Independent;
	// Pomiar jednego kanalu, wylacz opcje skanowania
	ADC_InitStruct.ADC_ScanConvMode = ENABLE;
	// Wlacz pomiar w trybie ciaglym
	ADC_InitStruct.ADC_ContinuousConvMode = ENABLE;
	// Nie bedzie wyzwalania zewnetrznego
	ADC_InitStruct.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	// Dane wyrownane do prawej - znaczacych bedzie 12 mlodszych bitow
	ADC_InitStruct.ADC_DataAlign = ADC_DataAlign_Right;
	// Dwa kanaly
	ADC_InitStruct.ADC_NbrOfChannel = 2;
	// Inicjuj przetwornik
	ADC_Init(ADC1, &ADC_InitStruct);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_71Cycles5); 
	ADC_RegularChannelConfig(ADC1, ADC_Channel_16, 2, ADC_SampleTime_239Cycles5); 

	// Wlaczenie czujnika temperatury   
  	ADC_TempSensorVrefintCmd(ENABLE);
	// Wlaczenie DMA
	ADC_DMACmd(ADC1, ENABLE);
	// Wlacz ADC1
	ADC_Cmd(ADC1, ENABLE);								

	// Resetuj rejestry kalibracyjne  
	ADC_ResetCalibration(ADC1);
	// Czekaj, az skonczy resetowac
	while(ADC_GetResetCalibrationStatus(ADC1));

	// Start kalibracji ADC1
	ADC_StartCalibration(ADC1);
	// Czekaj na zakonczenie kalibracji ADC1
	while(ADC_GetCalibrationStatus(ADC1));

  	LCD_Init();
 	LCD_Clear();

	// Start przetwarzania
	ADC_SoftwareStartConvCmd(ADC1, ENABLE); 

  	while(1)
  	{
		// Odczyt temperatury i napiecie + obliczenia
		temperatura = (1430 - ADCVal[1]*0.805) / 4.3 + 25;
		napiecie = ADCVal[0] * 0.807;   
		// Zapisanie wynikow pomiarow do tablic znakow
		sprintf(wynik_napiecie, "%d,%03d V", napiecie / 1000, napiecie % 1000);
		sprintf(wynik_temperatura, "T=%d stC", temperatura);

		LCD_GoTo(0,0);
		LCD_SendText((uint8_t*)wynik_napiecie);
		LCD_GoTo(1,0);
		LCD_SendText((uint8_t*)wynik_temperatura);
		delay_ms(500);  // Odswierzanie co okolo 500ms
		LCD_Clear();
  	}	
}

void DMA_Conf(void)
{
	DMA_InitTypeDef DMA_InitStruct;

	DMA_DeInit(DMA1_Channel1);
	DMA_InitStruct.DMA_PeripheralBaseAddr = ADC1_DR_Address;
	DMA_InitStruct.DMA_MemoryBaseAddr = (u32)&ADCVal;
	// Kierunek: zrodlem jest ADC
	DMA_InitStruct.DMA_DIR = DMA_DIR_PeripheralSRC;

	// Rozmiar burora: dwa kanaly = rozmiar bufora 2
	DMA_InitStruct.DMA_BufferSize = 2;

	DMA_InitStruct.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStruct.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStruct.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStruct.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	// Dane beda przesylane ciagle
	DMA_InitStruct.DMA_Mode = DMA_Mode_Circular;

	DMA_InitStruct.DMA_Priority = DMA_Priority_High;
	DMA_InitStruct.DMA_M2M = DMA_M2M_Disable;

	DMA_Init(DMA1_Channel1, &DMA_InitStruct);
	// Wlacz DMA
	DMA_Cmd(DMA1_Channel1, ENABLE);
}

void GPIO_Conf(void)
{
	GPIO_InitTypeDef GPIO_InitStruct;

	// Pomiar bedzie na wyprowadzeniu PC4
	GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_4;	 
	// Doprowadzenia jako analogowe wejscie
  	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AIN;
	// Inicjalizacja PC4	
  	GPIO_Init(GPIOC, &GPIO_InitStruct);	 
}

void RCC_Conf(void)
{  
  ErrorStatus HSEStartUpStatus; 

  // Reset ustawien RCC
  RCC_DeInit();

  // Wlacz HSE
  RCC_HSEConfig(RCC_HSE_ON);

  // Czekaj za HSE bedzie gotowy
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    // zwloka dla pamieci Flash
    FLASH_SetLatency(FLASH_Latency_2);
 	
    // HCLK = SYSCLK
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    // PCLK2 = HCLK
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    // PCLK1 = HCLK/2
    RCC_PCLK1Config(RCC_HCLK_Div2);

    // ADCCLK = PCLK2/4
    RCC_ADCCLKConfig(RCC_PCLK2_Div4); 

    // PLLCLK = 8MHz * 7 = 56 MHz
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_7);

    // Wlacz PLL 
    RCC_PLLCmd(ENABLE);

    // Czekaj az PLL poprawnie sie uruchomi
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    // PLL bedzie zrodlem sygnalu zegarowego
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    // Czekaj az PLL bedzie sygnalem zegarowym systemu
    while(RCC_GetSYSCLKSource() != 0x08);

	// Wlacz sygnal zegarowy dla ADC1, GPIOC i funkcji alternatywnych
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);

  	// Wlacz SPI (dla LCD)
  	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

	// Wlacz DMA
  	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
  }
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  // Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  // W przeciwnym wypadku ustaw na 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
