#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

ADC_InitTypeDef ADC_InitStructure;
TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;

uint16_t ADC1Val = 0;
char wynik[7];

int main(void)
{	
	#ifdef DEBUG
  		debug();
	#endif	
	
  	RCC_Conf();   
  	NVIC_Conf();
	GPIO_Conf();

  	// Ustawienia ukladu podstawy czasu
  	TIM_TimeBaseStructure.TIM_Period = 49999;
  	TIM_TimeBaseStructure.TIM_Prescaler = 5599;  //fclk = 56MHz/5600 = 10kHz
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  	// Konfiguracja kanalu 1
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  	TIM_OCInitStructure.TIM_Pulse = 49999;	// t=(49999 + 1)/10kHz=5s
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  	TIM_OC1Init(TIM1, &TIM_OCInitStructure);
  
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	// Pomiar jednego kanalu, wylacz opcje skanowania
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	// Wylacz pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	// Wyzwalanie z TIM1
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_CC1;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;

	ADC_Init(ADC1, &ADC_InitStructure);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_71Cycles5); 

	// Wlacz ADC1
	ADC_Cmd(ADC1, ENABLE);								
  
	ADC_ResetCalibration(ADC1);
	while(ADC_GetResetCalibrationStatus(ADC1));

	ADC_StartCalibration(ADC1);
	while(ADC_GetCalibrationStatus(ADC1));

  	LCD_Init();
 	LCD_Clear();

	// Wlaczenie timera i przerwania od CC1
	TIM_ITConfig(TIM1, TIM_IT_CC1, ENABLE);
  	TIM_Cmd(TIM1, ENABLE);

	// Start przetwarzania
	ADC_SoftwareStartConvCmd(ADC1, ENABLE); 
	
  	while(1)
  	{
		ADC1Val = ADC_GetConversionValue(ADC1) * 0.807;   

		sprintf(wynik, "%d,%03d V", ADC1Val / 1000, ADC1Val % 1000);

		LCD_Clear();
		LCD_GoTo(0,0);
		LCD_SendText("NAPIECIE NA P1:");
		LCD_GoTo(1,3);
		LCD_SendText((uint8_t*)wynik);
		delay_ms(500);
  	}	
}

void GPIO_Conf(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// Pomiar bedzie na wyprowadzeniu PC4
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;	 
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	
  	GPIO_Init(GPIOC, &GPIO_InitStructure);	 
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// ADCCLK = PCLK2/4
    	RCC_ADCCLKConfig(RCC_PCLK2_Div4); 

    	// PLLCLK = 8MHz * 7 = 56 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_7);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

		// Wlacz sygnal zegarowy dla ADC1, GPIOC i funkcji alternatywnych
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_AFIO, ENABLE);

  		// Wlacz SPI (dla LCD)
  		RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

		// Wlacz DMA
  		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

		RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
  	}
}

void NVIC_Conf(void)
{
  	NVIC_InitTypeDef NVIC_InitStructure;
#ifdef  VECT_TAB_RAM  
  	// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  	// W przeciwnym wypadku ustaw na 0x08000000
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif

  	// Wlacz przerwanie od TIM1
  	NVIC_InitStructure.NVIC_IRQChannel = TIM1_CC_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
