#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

ADC_InitTypeDef ADC_InitStructure;

uint16_t ADC1Val = 0;
char wynik[7];

int main(void)
{	
	#ifdef DEBUG
  		debug();
	#endif	
	
  	RCC_Conf();   
  	NVIC_Conf();
	GPIO_Conf();
  
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	// Wyzwalanie z EXTI11
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_Ext_IT11_TIM8_TRGO;

	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;

	ADC_Init(ADC1, &ADC_InitStructure);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_71Cycles5); 

	// Wlacz ADC1
	ADC_Cmd(ADC1, ENABLE);								
  
	ADC_ResetCalibration(ADC1);
	while(ADC_GetResetCalibrationStatus(ADC1));

	ADC_StartCalibration(ADC1);
	while(ADC_GetCalibrationStatus(ADC1));

  	LCD_Init();
 	LCD_Clear();

	ADC_SoftwareStartConvCmd(ADC1, ENABLE); 
	// Wlaczenie zewnetrznego wyzwalania
	ADC_ExternalTrigConvCmd(ADC1, ENABLE);
	
  	while(1)
  	{
		ADC1Val = ADC_GetConversionValue(ADC1) * 0.807;   
		sprintf(wynik, "%d,%03d V", ADC1Val / 1000, ADC1Val % 1000);

		LCD_Clear();
		LCD_GoTo(0,0);
		LCD_SendText("NAPIECIE NA P1:");
		LCD_GoTo(1,3);
		LCD_SendText((uint8_t*)wynik);
		delay_ms(500);
  	}	
}

void GPIO_Conf(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;

	// Pomiar bedzie na wyprowadzeniu PC4
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_11;	 
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	
  	GPIO_Init(GPIOC, &GPIO_InitStructure);	
	
  	// Poinformowanie uC o zrodle przerwania
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource11);

  	// Bedzie generowane przerwanie na zboczu opadajacym na EXTI_Line9
  	EXTI_InitStructure.EXTI_Line = EXTI_Line11;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Event;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);		 
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// ADCCLK = PCLK2/4
    	RCC_ADCCLKConfig(RCC_PCLK2_Div4); 

    	// PLLCLK = 8MHz * 7 = 56 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_7);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

		// Wlacz sygnal zegarowy dla ADC1, GPIOC i funkcji alternatywnych
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_AFIO, ENABLE);

  		// Wlacz SPI (dla LCD)
  		RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
  	}
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  	// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  	// W przeciwnym wypadku ustaw na 0x08000000
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
