#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>

SPI_InitTypeDef   SPI_InitStructure;

uint16_t TC77_temperatura, temperatura_temp;
char temperatura[4];
ErrorStatus HSEStartUpStatus;

void RCC_Conf(void);
void GPIO_Conf(void);
void NVIC_Conf(void);

int main(void)
{
#ifdef DEBUG
  debug();
#endif
  	RCC_Conf(); NVIC_Conf(); GPIO_Conf();

  	// Konfiguracja SPI1
  	SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Rx;
  	SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
  	SPI_InitStructure.SPI_DataSize = SPI_DataSize_16b;

  	SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  	SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;

	// NSS sprzetowo
  	SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;

  	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256;
  	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  	SPI_InitStructure.SPI_CRCPolynomial = 7;
  	SPI_Init(SPI1, &SPI_InitStructure);

  	// Wlacz NSS
  	SPI_SSOutputCmd(SPI1, ENABLE);

  	// Wlacz SPI1
  	SPI_Cmd(SPI1, ENABLE);

	LCD_Init();
	LCD_Clear();

	while (1)
  	{
		// SPI w tryb master = NSS w stan niski
		SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  		SPI_Init(SPI1, &SPI_InitStructure);

		// Czekaj na dane
    	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
    	// Odczytaj dane
    	TC77_temperatura = SPI_I2S_ReceiveData(SPI1) >> 3;

		// SPI w tryb slave = NSS w stan wysoki
		SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
  		SPI_Init(SPI1, &SPI_InitStructure);

		LCD_Clear();
		LCD_GoTo(0,0);
		// Przelicz i wyswietl temperature
		temperatura_temp = TC77_temperatura * 6;
		sprintf(temperatura, "%d,%01d", temperatura_temp / 100, (temperatura_temp % 100)/10 );

		LCD_SendText((u8*)temperatura);
		delay_ms(1000);
  	}
}

void RCC_Conf(void)
{
  	RCC_DeInit();
  	RCC_HSEConfig(RCC_HSE_ON);
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if (HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
    	FLASH_SetLatency(FLASH_Latency_2);
    	RCC_HCLKConfig(RCC_SYSCLK_Div1);
    	RCC_PCLK2Config(RCC_HCLK_Div2);
    	RCC_PCLK1Config(RCC_HCLK_Div2);
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);
    	RCC_PLLCmd(ENABLE);
    	while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
    	while (RCC_GetSYSCLKSource() != 0x08);
  	}

  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1, ENABLE);
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// SPI1 : SCK, MISO and MOSI 
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else 
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{
  while (1);
}
#endif
