#include "stm32f10x.h"
#include "lcd4bit.h"
#include <stdio.h>
#include <string.h>

USART_InitTypeDef USART_InitStructure;
GPIO_InitTypeDef GPIO_InitStructure;

char N[12];
char E[13];
char temp[5];

uint8_t i;
char GPGGA[] = "GPGGA";

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

char getch(void)
{
	char tmp;
	while(USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET);
	tmp = USART_ReceiveData(USART1); 
	return tmp;	
}

int main(void)
{
	#ifdef DEBUG
  		debug();
	#endif

  	RCC_Conf(); NVIC_Conf();

  	// Konfiguracja PA9 jako Tx
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  	// Konfiguracja PA10 jako Rx
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// Konfiguracja USART
  	USART_InitStructure.USART_BaudRate = 4800;
  	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  	USART_InitStructure.USART_StopBits = USART_StopBits_1;
  	USART_InitStructure.USART_Parity = USART_Parity_No;
  	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  	USART_InitStructure.USART_Mode = USART_Mode_Rx;	
	USART_Init(USART1, &USART_InitStructure);

	USART_Cmd(USART1, ENABLE);

  	LCD_Init();
  	LCD_GoTo(0,0);

  	while (1)
	{
		while( ( temp[0] = getch() ) != '$');
		for(i = 0; i < 5 ; i++)
			temp[i] = getch();
		if( strncmp(temp, GPGGA, 5) )
		{
			for(i = 0; i < 14 ; i++)
				temp[0] = getch();

			for(i = 0; i < 11; i++)
				N[i] = getch();
			N[i+1] = '\0';

			temp[0] = getch();

			for(i = 0; i < 12; i++)
				E[i] = getch();
			E[i+1] = '\0';
			
			LCD_GoTo(0,0);
			if(N[10] == 'N')
			{
				LCD_Clear(); 
				LCD_SendText((u8*)N); 
			}
			LCD_GoTo(1,0);
			if(E[11] == 'E')
				LCD_SendText((u8*)E);
		}		  
	}
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/4
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

  		// Wlacz taktowanie GPIOA
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_AFIO, ENABLE);
  	}
}

void NVIC_Conf(void)
{
	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1);
}
#endif
