#include "stm32f10x.h"
#include <stdio.h>

USART_InitTypeDef USART_InitStructure;
uint8_t przesuniecie, i, on, odebrano_polecenie, RxBuf[10];
uint16_t stan_portu;

void RCC_Conf(void);
void NVIC_Conf(void);
void GPIO_Conf(void);

void delay(int del)
{
	int i, tmp;
	tmp = 9000*del;
	for(i=0;i<tmp;i++);
}

char TxBuf[15] = {'o','n','L','E','D',':',' '};

int main(void)
{
	#ifdef DEBUG
  		debug();
	#endif

  	RCC_Conf();
  	NVIC_Conf();
  	GPIO_Conf();
	
	// Konfiguracja USART
  	USART_InitStructure.USART_BaudRate = 9600;
  	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  	USART_InitStructure.USART_StopBits = USART_StopBits_1;
  	USART_InitStructure.USART_Parity = USART_Parity_No;
  	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	
	
	USART_Init(USART1, &USART_InitStructure);

	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
	USART_Cmd(USART1, ENABLE);

  	while (1)
	{
		if(odebrano_polecenie)
		{
			if(RxBuf[0] == 'n')
				GPIO_SetBits(GPIOC, 1 << (RxBuf[1] - '0'));
			if(RxBuf[0] == 'f')
				GPIO_ResetBits(GPIOC, 1 << (RxBuf[1] - '0'));
			odebrano_polecenie = 0;
		}
		if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0))
		{
			stan_portu = ((GPIO_ReadInputData(GPIOC) >> 6) & 0x0F );
			przesuniecie = 7;
			for(i = 0; i<8;i++)
			{
				if((stan_portu >> i) & 0x01)
				{
					TxBuf[przesuniecie++] =  i + '1';
					TxBuf[przesuniecie++] =  ',';
				}
			}	 	 
			TxBuf[przesuniecie] = 0x0D;	
			USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
			delay(500);
		}
	}
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  		{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
   		// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/4
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

  		// Wlacz taktowanie GPIOA
  		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);
  	}
}

void GPIO_Conf(void)
{
  	GPIO_InitTypeDef GPIO_InitStructure;

  	// Konfiguracja PA0
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOC, &GPIO_InitStructure);


  	// Konfiguracja PA0
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  	// Konfiguracja PA9 jako Tx
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  	// Konfiguracja PA010 jako Rx
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;

  	GPIO_Init(GPIOA, &GPIO_InitStructure);

}

void NVIC_Conf(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

	#ifdef  VECT_TAB_RAM  
  		// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  		NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
	#else  // VECT_TAB_FLASH
  		// W przeciwnym wypadku ustaw na 0x08000000
  		NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	#endif

  	// Wybranie grupy priorytetw
 	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  	// Wlacz przerwanie od USART1
  	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1);
}
#endif
