#include "stm32f10x.h"
#include "fatfs/src/ff.h"
#include "fatfs/src/diskio.h"

void RCC_Conf(void);
void NVIC_Conf(void);
void SysTick_Conf(void);

static FATFS g_sFatFs;
static DIR g_sDirObject;
static FILINFO g_sFileInfo;
static FIL g_sFileObject;

#define PATH_BUF_SIZE   80
static char g_cCwdBuf[PATH_BUF_SIZE] = "/";

int main(void)
{
	FRESULT fresult;
	FIL plik;
	FILINFO plikInfo ;
	DIR Dir;
	FILINFO Info[1000];
	FILINFO *p;

	WORD zapisanych_bajtow;

  	RCC_Conf(); SysTick_Conf();

    fresult = f_mount(0, &g_sFatFs);

	// Tworzenie pliku
	fresult = f_open (&plik,"plik.txt", FA_CREATE_ALWAYS);
	fresult = f_close (&plik);

   	// Tworzenie katalogu
	fresult = f_mkdir("katalog1");

	// Zapis pliku
	fresult = f_open (&plik,"plik.txt", FA_WRITE);
	fresult = f_write(&plik, "zawartosc pliku", 15, &zapisanych_bajtow);
	fresult = f_close (&plik);

	// Usuniecie pliku
	fresult = f_unlink("plik.txt");

	p = Info;

    fresult = f_opendir(&Dir, "katalog1");

    for(;;)
    {
        fresult = f_readdir(&Dir, &plikInfo);

        if(fresult != FR_OK)
            return(fresult);

        if(!plikInfo.fname[0])
            break;

		// p jest wkaznikiem na tablice elemetnow typu FILINFO
		*p++ = plikInfo;
    } 

  	while (1);
}

void RCC_Conf(void)
{  
  	ErrorStatus HSEStartUpStatus; 

  	// Reset ustawien RCC
  	RCC_DeInit();

  	// Wlacz HSE
  	RCC_HSEConfig(RCC_HSE_ON);

  	// Czekaj za HSE bedzie gotowy
  	HSEStartUpStatus = RCC_WaitForHSEStartUp();

  	if(HSEStartUpStatus == SUCCESS)
  	{
    	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    	// zwloka dla pamieci Flash
    	FLASH_SetLatency(FLASH_Latency_2);
 	
    	// HCLK = SYSCLK
    	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    	// PCLK2 = HCLK
    	RCC_PCLK2Config(RCC_HCLK_Div1); 

    	// PCLK1 = HCLK/2
    	RCC_PCLK1Config(RCC_HCLK_Div2);

    	// PLLCLK = 8MHz * 9 = 72 MHz
    	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    	// Wlacz PLL 
    	RCC_PLLCmd(ENABLE);

    	// Czekaj az PLL poprawnie sie uruchomi
    	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    	// PLL bedzie zrodlem sygnalu zegarowego
    	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    	// Czekaj az PLL bedzie sygnalem zegarowym systemu
    	while(RCC_GetSYSCLKSource() != 0x08);

		// Wlacz sygnal zegarowy dla GPIOC
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE); // Wlacz sygnal zegarowy dla GPIOC
  	}
}

void SysTick_Conf(void)
{   
  	if (SysTick_Config(SystemFrequency / 100))
  	{
  		// W przypadku bledu konfiguracji peca w nieskonczonej petli
		while(1);
  	}					
}

void NVIC_Conf(void)
{
#ifdef  VECT_TAB_RAM  
  	// Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  // VECT_TAB_FLASH
  	// W przeciwnym wypadku ustaw na 0x08000000
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
void assert_failed(uint8_t* file, uint32_t line)
{ 
  	while (1);
}
#endif
